//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

package com.ibm.as400.opnav.MsgQueueSample1;

// Import the entire opnav package
import com.ibm.as400.opnav.*;

// Import the PDML runtime
import com.ibm.as400.ui.framework.java.*;

// Import the access classes we need
import com.ibm.as400.access.AS400;
import com.ibm.as400.access.QSYSObjectPathName;

// Miscellaneous Sun classes
import javax.swing.JOptionPane;
import java.awt.Frame;
import java.text.MessageFormat;

/**
 * The <code>ActionsManager</code> implementation for this plug-in.
 * <p>
 * An <code>ActionsManager</code> implementation provides Operations
 * Navigator with lists of possible user actions for the plug-in's objects.
 * If the user selects an action, <code>ActionsManager</code> is called
 * to carry out the action.
 * See <strong>com.ibm.as400.opnav.ActionsManager.</strong>
 * <p>
 * <code>MqActionsManager</code> is invoked whenever the Navigator
 * needs to display the context menu for a message queue container,
 * or a message object in the container.  It will also be called to
 * validate the Navigator's toolbar buttons whenever the user clicks on the
 * container or one of its objects.
 */
public class MqActionsManager implements ActionsManager
{
    /**
     * The list of selected objects.
     */
    ObjectName[]    m_objectNames;

    /**
     * The target of a right mouse button drop, if this
     * is a drag/drop context menu.
     */
    ObjectName      m_dropTarget;

    /**
     * The owning frame for any dialog windows displayed
     * while processing an action.
     */
    Frame           m_owner;

    // Use the ResourceLoader for loading MRI from the sample resource bundle
    static private ResourceLoader  m_loader = new ResourceLoader();

    static
    {
        // Set up to load MRI.  We do this in the static initializer for
        // the class so that we only load the resource bundle once.
        m_loader.setResourceName("com.ibm.as400.opnav.MsgQueueSample1.MessageQueueGUI");

        // Set the service log file name in production
//      try { Monitor.setLogFileName(Monitor.SHARED_LOG_FILE_NAME); }
        try { Monitor.setLogFileName(null); }
        catch (java.io.IOException e) {}
    }

    /**
     * Identifies the objects in a list on which the user may wish
     * to perform some action.
     *
     * Called immediately after this object is instantiated.
     * <p>
     * If Operations Navigator is displaying the list of available actions
     * resulting from a drag and drop operation, the <code>dropTarget</code>
     * parameter identifies the name of the target container.  Otherwise,
     * <code>dropTarget</code> will be null.
     * See <strong>com.ibm.as400.opnav.ActionsManager.</strong>
     *
     * @param objectNames	An array of object names.
     * @param dropTarget	The container into which the objects are being dropped (may be null).
     */
    public void initialize(ObjectName[] objNames, ObjectName dropTarget)
    {
        // Save the array of selected object names
        m_objectNames = objNames;

        // Save the drop target in case we need to handle right mouse button drops (not implemented)
        m_dropTarget = dropTarget;
    }

    /**
     * Returns a list of actions which are applicable to the specified objects.
     *
     * Called when Operations Navigator needs to display the available
     * actions on one or more objects in a list of system objects.
     * See <strong>com.ibm.as400.opnav.ActionsManager.</strong>
     *
     * @param flags  The constants which indicate the type of actions requested.
     * @return    A list of action descriptors, each of which describes one action.
     * If no actions are defined on the objects, <code>queryActions</code> should
     * return an array of length zero.
     */
    public ActionDescriptor[] queryActions(int flags)
    {
        // Default is to return an array with zero elements
        ActionDescriptor[] actions = new ActionDescriptor[0];

        // Determine what type of object(s) we're dealing with
        String objectType = null;

        // NOTE: We assume all objects are of the same type
        try { objectType = m_objectNames[0].getObjectType(); }
        catch (ObjectNameException e)
        {
            // Error unlikely in production
            Monitor.logThrowable(e);

            return actions;
        }

        // Handle the normal context menu case
        if (objectType.equals("Sample1"))
        {
            // This is a message queue container.  We have stuff to add to
            // the "creation" section of the menu (the cascading New menu).

            if ((flags & CREATION_ACTIONS) == CREATION_ACTIONS)
            {
                // Create an array to contain the returned descriptor
                actions = new ActionDescriptor[1];

                // Create the descriptor for "Message"
                ActionDescriptor d;

                d = new ActionDescriptor(1);
                d.setText(m_loader.getString("menu.text.new"));
                d.setHelpText(m_loader.getString("menu.help.new"));
                d.setVerb("NEWMSG");

                // Set it in the array
                actions[0] = d;
            }
        }

        if (objectType.equals("Message1"))
        {
            // This is a message object.  We want to add stuff to
            // the "standard" section of the menu.

            if ((flags & STANDARD_ACTIONS) == STANDARD_ACTIONS)
            {
                // Create an array to contain the returned descriptor
                actions = new ActionDescriptor[1];

                // Create the descriptor for Delete
                ActionDescriptor d;

                d = new ActionDescriptor(2);
                d.setText(m_loader.getString("menu.text.delete"));
                d.setHelpText(m_loader.getString("menu.help.delete"));
                d.setVerb("DELETE");

                // Set it in the array
                actions[0] = d;
            }
        }

        // Return actions, if any
        return actions;
    }

    /**
     * Informs the <code>ActionsManager</code> implementation that a particular
     * action has been selected by the user.
     * <p>
     * The implementation should respond to the request by displaying a
     * UI panel or taking some action on behalf of the user.
     *
     * @param action  The identifier of the action specified in the
     * <code>ActionDescriptor</code> that was returned by <code>queryActions</code>.
     * @param owner   The frame to be specified as the owner of any dialog panels that are displayed.
     */
    public void actionSelected(int action, Frame owner)
    {
        // Set the owning frame for later use
        m_owner = owner;

        switch (action)
        {
            case 1:
                // Perform the Add action
                promptNewMessage();
                break;

            case 2:
                // Perform the Delete action
                confirmDeleteMessages();
                break;

            default:
                // Error unlikely in production
                Monitor.logError("MqActionsManager: Invalid action ID: " + action);
                break;
        }
    }

    private void promptNewMessage()
    {
        // If an error occurs we need the name of the AS/400 we're trying to connect to
        String systemName = "UNKNOWN";

        // Get the name of the AS/400 associated with this container object
        try { systemName = m_objectNames[0].getSystemName(); }
        catch (ObjectNameException e) {}

        // Get the Toolbox object for the container's parent AS/400 system
        AS400 as400 = null;

        try { as400 = (AS400)m_objectNames[0].getSystemObject(); }
        catch (ObjectNameException e)
        {
            // Log the error
            Monitor.logThrowable(e);

            // Load the error message text
            String msg = m_loader.getString("error.text.connection.info");

            // Substitution text is the system name
            Object[] args = { systemName };

            // Format the error message
            String usrMsg = MessageFormat.format(msg, args);

            // Display it to the user
            MessageBoxDialog.showMessageDialog(m_owner,
                                               usrMsg,
                                               m_loader.getString("error.title.msgbox"),
                                               JOptionPane.WARNING_MESSAGE);

            return;
        }

        // Instantiate the data bean for the New Message panel
        MqNewMessageBean msgBean = new MqNewMessageBean(as400, m_owner);
        msgBean.load();

        // Set up to pass to PanelManager
        DataBean[] dataBeans = { msgBean };

        // Create the panel
        PanelManager pm = null;
        try
        {
            pm = new PanelManager("com.ibm.as400.opnav.MsgQueueSample1.MessageQueueGUI",
                                  "IDD_MSGQ_ADD",
                                  dataBeans,
                                  m_owner);
        }
        catch (DisplayManagerException e)
        {
            // Tell the user an error occurred
            e.displayUserMessage(m_owner);

            return;
        }

        // Display the panel (we give up control here)
        pm.setVisible(true);

        // If no new message was created, simply return
        if (!msgBean.m_actionPerformed)
            return;

        // Refresh the list view to show the new added message
        try { new UIServices().refreshList(m_owner, m_loader.getString("message.text.newmsg")); }
        catch (UIServicesException e)
        {
            // Error unlikely
            Monitor.logThrowable(e);
        }
    }

    private void confirmDeleteMessages()
    {
        // Build a list of the selected objects
        MqMessage[] objList = new MqMessage[m_objectNames.length];

        // Get the list objects for the items that are selected
        for (int i = 0; i < m_objectNames.length; i++)
        {
            MqMessage msg = null;

            try { msg = (MqMessage)m_objectNames[i].getListObject(); }
            catch (ObjectNameException e)
            {
                // Detailed exception was already logged by MessagesListManager
                Monitor.logError(e.getMessage());

                // Load the error message text
                String usrMsg = m_loader.getString("error.text.program");

                // Display it to the user
                MessageBoxDialog.showMessageDialog(m_owner,
                                                   usrMsg,
                                                   m_loader.getString("error.title.msgbox"),
                                                   JOptionPane.WARNING_MESSAGE);

                return;
            }

            objList[i] = msg;
        }

        // Pass the list to the data bean for the delete dialog
        MqDeleteMessageBean bean = new MqDeleteMessageBean(objList, m_owner);
        bean.load();

        // Set up to pass the bean to the PDML framework
        DataBean[] dataBeans = { bean };

        // Create the panel
        PanelManager pm = null;
        try
        {
            pm = new PanelManager("com.ibm.as400.opnav.MsgQueueSample1.MessageQueueGUI",
                                  "IDD_MSGQ_CONFIRM_DELETE",
                                  dataBeans,
                                  m_owner);
        }
        catch (DisplayManagerException e)
        {
            Monitor.logThrowable(e);
            return;
        }

        // Display the confirmation dialog
        pm.setVisible(true);

        // Create the UIServices object needed to refresh the display
        UIServices services = new UIServices();

        if (!bean.m_actionPerformed)
        {
            // Indicate that no messages were deleted in the OpNav status area
            String message = m_loader.getString("message.text.nodeletemsg");

            try { services.updateStatusArea(m_owner, message); }
            catch (UIServicesException e)
            {
                // Error unlikely
                Monitor.logThrowable(e);
            }

            return;
        }

        // Load the status area text
        String message = m_loader.getString("message.text.deletemsg");

        // Substitution text is the count of deleted messages and the total count
        Object[] countString = { new Integer(bean.m_deleteCount), new Integer(m_objectNames.length) };

        // Format the message
        message = MessageFormat.format(message, countString);

        // Refresh the list to show the new message and update the status area
        try { services.refreshList(m_owner, message); }
        catch (UIServicesException e)
        {
            // Error unlikely
            Monitor.logThrowable(e);
        }
    }
}
