//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

package com.ibm.as400.opnav.MsgQueueSample2;

// Import the entire OpNav package
import com.ibm.as400.opnav.*;

/**
 * The master <code>ListManager</code> implementation for the
 * sample plug-in. This class is identified in the Windows registry.
 * <p>
 * A <code>ListManager</code> implementation provides list data for 
 * Operations Navigator tree and list views.  Its methods are called
 * whenever the Navigator needs to display the contents of a container.
 * See <strong>com.ibm.as400.opnav.ListManager.</strong>
 * <p>
 * A new instance of <code>MqListManager</code> is created for each
 * container selected by the user. Based on the type of container
 * selected, this object instantiates another <code>ListManager</code>
 * implementation to do the actual work. Subsequent method calls delegate
 * to this "slave" implementation.
 * 
 */
public class MqListManager implements ListManager
{
    /**
     * Identifies the ListManager implementation that does the actual work.
     */
    ListManager m_delegate;

    /**
     * Identifies the container to be enumerated.
     *
     * Called immediately after the receiver is instantiated.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param container  The container whose contents the user wishes to display.
     */
    public void initialize(ObjectName container)
    {
		try
		{
			// Figure out which ListManager implementation is required
			if (container.getObjectType().equals("Sample2"))
				m_delegate = new MqMessageQueuesListManager();

            else if (container.getObjectType().equals("MessageQueue2"))
				m_delegate = new MqMessagesListManager();
			
            else
				throw new IllegalArgumentException(container.toString());
		}
		catch (ObjectNameException e)
		{
            // Error unlikely in production
            Monitor.logThrowable(e);
		}

		// Now forward the initialize request
		m_delegate.initialize(container);
    }

    /**
     * Prepares the list for enumeration.
     *
     * Called when the user initially selects a container.
     * Once it has been called, <code>getStatus</code> should return a
     * value other than <code>LIST_CLOSED</code>.
     * <p>
     * <code>open</code> is called on a data thread so that long running
     * operations will not degrade the performance of the user interface.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     */
    public void open()
    {
        // Forward the request
        m_delegate.open();
    }

    /**
     * Returns an error message.
     *
     * Called after <code>getListStatus</code> has returned
     * a value of <code>LIST_ERROR</code>.  The returned message should be
     * in a form suitable for displaying to the user.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @return  A user error message, or <code>null</code> if no error message is available.
     */
    public String getErrorMessage()
    {
        // Forward the request
		return m_delegate.getErrorMessage();
    }

    /**
     * Returns the current status of the list.
     *
     * Called after the list has been opened to learn
     * whether the open attempt was successful.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @return  A status constant which reflects the current list status.
     */
    public int getStatus()
    {
        // Forward the request
        return m_delegate.getStatus();
    }

    /**
     * Returns the total count of objects in the list.
     * <p>
     * Called after the list has been successfully opened.
     * If <code>getStatus</code> returned a value of
     * <code>LIST_COMPLETE</code>, this method should return
     * the total number of objects in the list.  If <code>getStatus</code>
     * returned <code>LIST_INCOMPLETE</code>, this method should
     * return the number of objects currently known to be in the list.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @return  The number of objects in the list.
     */
    public int getItemCount()
    {
        // Forward the request
        return m_delegate.getItemCount();
    }

    /**
     * Returns the item identifier for a list object.
     * <p>
     * Called repeatedly when populating a view of a container.
     * Long running operations should be avoided.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param  index  The zero-based index of the item for which an
     * item identifier is requested.
     * @return  The item identifier that describes the object at the
     * specified position in the list.
     */
    public ItemIdentifier itemAt(int index)
    {
        // Forward the request
		return m_delegate.itemAt(index);
    }

    /**
     * Returns the attributes of a list object.
     * <p>
     * Called repeatedly when populating a view of a container.
     * Long running operations should be avoided.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param   item  The item identifier for the object whose attributes
     * are to be retrieved.
     * @return  The integer sum of the attribute constants.
     */
    public int getAttributes(ItemIdentifier item)
    {
        // Forward the request
        return m_delegate.getAttributes(item);
    }

    /**
     * Returns the image file name for a list object.
     * <p>
     * Called repeatedly when populating the view of a container.
     * Long running operations should be avoided.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param   item  The item identifier for the object whose image file
     * name is to be retrieved.
     * @param   flags The constants which reflect the status of the object.
     * @return  The name of the image file containing the icon for the object.
     */
    public String getImageFile(ItemIdentifier item, int flags)
    {
        // Forward the request
        return m_delegate.getImageFile(item, flags);
    }

    /**
     * Returns the icon index into the plug-in's resource DLL for a list object.
     * <p>
     * Called repeatedly when populating the view of a container.
     * Long running operations should be avoided.
     *
     * @param   item  The item identifier for the object whose icon index
     * is to be retrieved.
     * @param   flags The constants which reflect the status of the object.
     * @return  The index into the plug-in's resource DLL of the icon for the object.
     */
    public int getIconIndex(ItemIdentifier item, int flags)
    {
		// An explicit cast is required here since getIconIndex isn't
		// defined on the ListManager interface.

		if (m_delegate instanceof MqMessageQueuesListManager)
			return ((MqMessageQueuesListManager)m_delegate).getIconIndex(item, flags);

		if (m_delegate instanceof MqMessagesListManager)
			return ((MqMessagesListManager)m_delegate).getIconIndex(item, flags);

		return 0;
    }

    /**
     * Returns the column headings for this list.
     *
     * Called after the list has been successfully opened.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @return  A list of <code>ColumnDescriptor</code> objects
     * that describe the column headings for this list.
     */
    public ColumnDescriptor[] getColumnInfo()
    {
        // Forward the request
        return m_delegate.getColumnInfo();
	}

    /**
     * Returns the data for a list column.
     * <p>
     * Called repeatedly when populating the view of a container.
     * Long running operations should be avoided.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param   item  The item identifier for the object whose column
     * data is to be retrieved.
     * @param   columnID  The numeric ID that identifies the column.
     * @return  The data for the specified column converted to a String.
     */
    public String getColumnData(ItemIdentifier item, int columnID)
    {
        // Forward the request
        return m_delegate.getColumnData(item, columnID);
    }

    /**
     * Returns a cached list object.
     *
     * Called when another interface implementation
     * requires access to the object that represents a
     * resource on the system being managed.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param objName The name which identifies the list object will be retrieved.
     * @return The cached list object.
     */
    public Object getListObject(ObjectName objName)
    {
        // Forward the request
        return m_delegate.getListObject(objName);
    }

    /**
     * Frees all list resources.
     *
     * Called when the user closes a view of a container,
     * or when a refresh of the container's contents is requested.
     * Once it has been called, <code>getStatus</code> should return a
     * value of <code>LIST_CLOSED</code>.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     */
    public void close()
    {
        // Forward the request
		m_delegate.close();
    }

    /**
     * Provides notification that the Operations Navigator application is terminating.
     *
     * Called when the user closes the application.
     * Provides the implementation with the opportunity to clean
     * up any connection handles and save any persistent data.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     */
    public void prepareToExit()
    {
        // Forward the request
		m_delegate.prepareToExit();
    }
}
