//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

// We avoid any references to the opnav package in case we want to
// use this class in a standalone application or applet

package com.ibm.as400.opnav.MsgQueueSample4;

// Import the PCML runtime
import com.ibm.as400.data.ProgramCallDocument;
import com.ibm.as400.data.PcmlException;

// Import the access classes we need
import com.ibm.as400.access.AS400;
import com.ibm.as400.access.MessageQueue;
import com.ibm.as400.access.QueuedMessage;
import com.ibm.as400.access.QSYSObjectPathName;

// Exceptions that the access package can throw
import com.ibm.as400.access.AS400Exception;
import com.ibm.as400.access.AS400SecurityException;
import com.ibm.as400.access.ConnectionDroppedException;
import com.ibm.as400.access.ErrorCompletingRequestException;
import com.ibm.as400.access.ObjectDoesNotExistException;
import com.ibm.as400.access.ServerStartupException;

import java.io.IOException;
import java.lang.InterruptedException;
import java.net.UnknownHostException;

// Miscellaneous Sun classes
import java.util.Enumeration;
import java.util.Vector;

/**
 * Encapsulates a list of messages on an AS/400 message queue.
 * <code>MqMessageQueue</code> can obtain a list of the messages,
 * and provides access to individual message objects.
 * <p>
 * <code>MqMessageQueue</code> uses the class
 * <code>com.ibm.as400.access.MessageQueue</code> provided in the
 * AS/400 Toolbox for Java to obtain the list of messages.
 */
public class MqMessageQueue
{
    /**
     * The system object corresponding to the AS/400 on which the messages reside.
     */
    AS400				m_as400;

    /**
     * The name of the message queue.
     */
	String				m_name;

    /**
     * The name of the library where the message queue resides.
     */
	String				m_library;

    /**
     * The text description of the message queue.
     */
	String				m_textDescription;

    /**
     * The list of stored message objects.
     */
    Vector				m_msgList = new Vector();

    /**
     * The property on which to sort the list of message objects.
     */
    String              m_sortProperty;

    /**
     * Indicates whether the messages should be sorted in ascending
     * or descending order.
     */
    String              m_sortOrder;

    /**
     * Constructs an <code>MqMessageQueue</code>.
     *
     * @param as400 The system object for the AS/400 on which the message queue resides.
     * See <strong>com.ibm.as400.access.AS400.</strong>
     */
    public MqMessageQueue(AS400 as400)
    {
        m_as400 = as400;
    }

    /**
     * Constructs an <code>MqMessageQueue</code>.
     *
     * @param name The name of the message queue.
     * @param library The library in which the message queue resides.
     * @param as400 The system object for the AS/400 on which the message queue resides.
     * See <strong>com.ibm.as400.access.AS400.</strong>
     */
    public MqMessageQueue(String name, String library, AS400 as400)
    {
        m_name = name;
        m_library = library;
        m_as400 = as400;
    }

    /**
     * Constructs an <code>MqMessageQueue</code> using the specified
     * <code>ProgramCallDocument.</code>  This constructor knows how to
     * extract the message queue parameters from a record returned by
     * a call to the QGYOLOBJ API.
     *
     * @param programName The name of the program call definition that was used to
     * retrieve the message queue data.
     * @param pcd The ProgramCallDocument object containing the message queue data.
     * @param index The index of the record into the ProgramCallDocument containing
     * the message queue data.
     * @param as400 The system object for the system on which the message queue resides.
     * See <strong>com.ibm.as400.access.AS400.</strong>
     * @exception PcmlException If an error occurred attempting to use the
     * program call specification for QGYOLOBJ. This specification may
     * be found in the file qgyolobj.pcml.
     */
	public MqMessageQueue(String programName, ProgramCallDocument pcd, int index, AS400 as400)
		throws PcmlException
	{
		// Store the system object
		m_as400 = as400;

		// Extract the message queue attributes
		int[] indexList = { index };

        // Store the message queue name
		m_name = (String)pcd.getValue(programName + ".receiverVariable.objectNameUsed", indexList);
		debug("Message queue: " + m_name);

        // Store the message queue library
		m_library = (String)pcd.getValue(programName + ".receiverVariable.objectLibraryNameUsed", indexList);
		debug("Message queue library: " + m_library);

        // Store the text description
		m_textDescription = (String)pcd.getValue(programName + ".receiverVariable.text.textDescription", indexList);
		debug("Message queue description: " + m_textDescription);

		debug("");
		debug("--------------------------------------------------------------");
		debug("");
	}

    /**
     * Returns the name of this message queue.
     *
     * @return The name of the message queue.
     */
    public String getName()
    {
        return m_name;
    }

    /**
     * Returns the library in which this message queue resides.
     *
     * @return The name of the message queue's library.
     */
    public String getLibrary()
    {
        return m_library;
    }

    /**
     * Returns the text description for this message queue.
     *
     * @return The text description for the message queue.
     */
    public String getTextDescription()
    {
        return m_textDescription;
    }

    /**
     * Obtains the list of messages for this message queue from the AS/400.
     * This method uses the Toolbox class <code>com.ibm.as400.access.MessageQueue</code>
     * to obtain the messages.
     *
     * @exception AS400Exception                  If the AS/400 system returns an error message.
     * @exception AS400SecurityException          If a security or authority error occurs.
     * @exception ConnectionDroppedException      If the connection is dropped unexpectedly.
     * @exception ErrorCompletingRequestException If an error occurs before the request is completed.
     * @exception InterruptedException            If this thread is interrupted.
     * @exception IOException                     If an error occurs while communicating with the AS/400.
     * @exception ObjectDoesNotExistException     If the AS/400 object does not exist.
     * @exception ServerStartupException          If the AS/400 server cannot be started.
     * @exception UnknownHostException            If the AS/400 system cannot be located.
     */
    public void loadAS400Messages()
        throws AS400Exception, AS400SecurityException, ConnectionDroppedException,
        ErrorCompletingRequestException, InterruptedException, IOException,
        ObjectDoesNotExistException, ServerStartupException, UnknownHostException
    {
		// Determine if a pathname is required
		String queueName;

		if (m_name != null)
		{
			QSYSObjectPathName queuePath = new QSYSObjectPathName(m_library, m_name, "MSGQ");
			queueName = queuePath.getPath();
		}
		else
			queueName = MessageQueue.CURRENT;

        // Instantiate the Toolbox MessageQueue object
        MessageQueue queue = new MessageQueue(m_as400, queueName);

        // Enumerate the messages
        Enumeration e = queue.getMessages();

        while (e.hasMoreElements())
        {
            // Get the next queued message
            QueuedMessage qm = (QueuedMessage)e.nextElement();

            // Create an instance of our message object
            MqMessage msg = new MqMessage(qm);

            // Insert the message into the list in the correct position
            if (m_sortProperty == null || m_msgList.size() == 0)
            {
                // Add this message to the list
                m_msgList.addElement(msg);
            }
            else
            {
                // Locate the correct position for the new message
                boolean found = false;

                for (int i = 0; i < m_msgList.size(); i++)
                {
                    MqMessage tmp = (MqMessage)m_msgList.elementAt(i);

                    if (m_sortProperty.equals("TEXT"))
                    {
                        if (m_sortOrder.equals("ASCENDING"))
                        {
                            if (msg.getText().compareTo(tmp.getText()) < 0)
                            {
                                m_msgList.insertElementAt(msg, i);
                                found = true;
                                break;
                            }
                        }
                        else
                        {
                            if (msg.getText().compareTo(tmp.getText()) > 0)
                            {
                                m_msgList.insertElementAt(msg, i);
                                found = true;
                                break;
                            }
                        }
                    }
                }

                // If a suitable position was not found,
                // add the element to the end of the list
                if (!found)
                    m_msgList.addElement(msg);
            }
        }
    }

    /**
     * Returns the count of retrieved messages.
     *
     * @return The count of messages on the message queue.
     */
    public int getMessageCount()
    {
        return m_msgList.size();
    }

    /**
     * Returns an AS/400 message from the <code>MqMessageQueue.</code>
     *
     * @param index The position of the desired message in the list.
     * @return The <code>MqMessage</code> at the specified index.
     */
    public MqMessage getMessage(int index)
    {
        return (MqMessage)m_msgList.elementAt(index);
    }

    /**
     * Returns the sort criteria for this list.
     *
     * @return A two-element array of strings.  The first element contains
     * the name of the property on which the list should be sorted.  The
     * second element contains the string "ASCENDING" if the list is to be sorted
     * in ascending order, or "DESCENDING" for descending order.
     */
    public String[] getSortParameters()
    {
        String[] result = { m_sortProperty, m_sortOrder };
        return result;
    }

    /**
     * Sets the sort criteria for this list.
     *
     * @param property The name of the property on which the list should be sorted.
     * @param sortOrder "ASCENDING" if the list is to be sorted in ascending order;
     * "DESCENDING" for descending order.
     */
    public void setSortParameters(String property, String sortOrder)
    {
        m_sortProperty = property;
        m_sortOrder = sortOrder;
    }

	private static boolean debugFlag = true;
	private static void debug(String text)
	{
		if (debugFlag)
			System.out.println("MqMessageQueue: " + text);
	}
}
