//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

// We avoid any references to the opnav package in case we want to
// use this class in a standalone application or applet

package com.ibm.as400.opnav.MsgQueueSample4;

// Import the access classes we need
import com.ibm.as400.access.AS400;
import com.ibm.as400.access.AS400Message;

// Import the PCML runtime
import com.ibm.as400.data.ProgramCallDocument;
import com.ibm.as400.data.PcmlException;

// Miscellaneous Sun classes
import java.util.Vector;

/**
 * Encapsulates a list of message queues on a particular AS/400.
 * <code>MqMessageQueueList</code> maintains a count of the message
 * queues, and provides access to individual queue objects.
 * <p>
 * Because there is no Toolbox class that returns a list of message queues,
 * this class uses the Program Call Markup Language (PCML) facility
 * provided with the AS/400 Toolbox for Java to invoke the AS/400 API QGYOLOBJ.
 * For more information see the AS/400 System API reference.
 */
public class MqMessageQueueList
{
    /**
     * The system object for the AS/400 on which the message queues reside.
     */
    AS400   m_as400;

    /**
     * The list of stored message queue objects.
     */
    Vector  m_msgQueueList = new Vector();

    /**
     * Constructs an <code>MqMessageQueueList</code>.
     *
     * @param as400 The system object for the AS/400 on which the message queues reside.
     * See <strong>com.ibm.as400.access.AS400.</strong>
     */
    public MqMessageQueueList(AS400 as400)
    {
        m_as400 = as400;
    }

    /**
     * Obtains the list of messages for this message queue from the AS/400.
     * This method uses the PCML runtime to invoke the QGYOLOBJ API.
     *
     * @exception PcmlException If an error occurred attempting to use the
     * program call specification for QGYOLOBJ. This specification may
     * be found in the file qgyolobj.pcml.
     */
    public void loadAS400MessageQueues()
        throws PcmlException
    {
		// Create the program call document object for QGYOLOBJ
		ProgramCallDocument pcd = new ProgramCallDocument(m_as400, "com.ibm.as400.opnav.MsgQueueSample4.qgyolobj");

		debug("Calling QGYOLOBJ...");

		// Call the primary API
		boolean succeeded = pcd.callProgram("qgyolobj");

		if (succeeded)
		{
			// Obtain the total record count
			Integer totalRecords = (Integer)pcd.getValue("qgyolobj.listInformation.totalRecords");
			int total = totalRecords.intValue();

			// Obtain the count of records returned
			Integer recordsReturned = (Integer)pcd.getValue("qgyolobj.listInformation.recordsReturned");
			int returned = recordsReturned.intValue();

			// Initialize processed count
			int processed = returned;

			debug("TOTAL RECORDS: " + total);
			debug("RECORDS PROCESSED: " + processed);

			// Instantiate a message queue object for each record
			for (int i = 0; i < returned; i++)
			{
				MqMessageQueue mq = new MqMessageQueue("qgyolobj", pcd, i, m_as400);
	            m_msgQueueList.addElement(mq);
			}

			// Get the rest of the records if required
			while (total - processed > 0)
			{
				debug("Calling QGYGTLE...");

				// Copy the request handle
				pcd.setValue("qgygtle.requestHandle", pcd.getValue("qgyolobj.listInformation.requestHandle"));

				// Always request the total record count
				pcd.setValue("qgygtle.numberOfRecordsToReturn", totalRecords);

				// Compute the starting record number (1 indexed)
				pcd.setValue("qgygtle.startingRecord", new Integer(total - (total - processed) + 1));

				// Call QGYGTLE to get the next batch of records
				succeeded = pcd.callProgram("qgygtle");

				if (succeeded)
				{
					// Obtain count of records returned
					recordsReturned = (Integer)pcd.getValue("qgygtle.listInformation.recordsReturned");
					returned = recordsReturned.intValue();

					// Increment count of processed records
					processed += returned;

					debug("TOTAL RECORDS: " + total);
					debug("RECORDS PROCESSED: " + processed);

					// Instantiate a message queue object for each record
					for (int i = 0; i < returned; i++)
					{
						MqMessageQueue mq = new MqMessageQueue("qgygtle", pcd, i, m_as400);
						m_msgQueueList.addElement(mq);
					}
				}
				else
				{
                    // Print out all of the AS/400 messages
					AS400Message[] msgs = pcd.getMessageList("qgygtle");

					for (int i = 0; i < msgs.length; i++)
					{
						System.err.print(msgs[i].getID() + "  ");
						System.err.println(msgs[i].getText());
					}
				}
			}

			// Finally, close the list
			debug("Calling QGYCLST...");

			// Copy the request handle
			pcd.setValue("qgyclst.requestHandle", pcd.getValue("qgyolobj.listInformation.requestHandle"));

			// Call QGYCLST
			succeeded = pcd.callProgram("qgyclst");

			if (succeeded)
				debug("TOTAL QUEUE COUNT: " + m_msgQueueList.size());
			else
			{
                // Print out all of the AS/400 messages
				AS400Message[] msgs = pcd.getMessageList("qgyclst");

				for (int i = 0; i < msgs.length; i++)
				{
					System.out.print(msgs[i].getID() + "  ");
					System.out.println(msgs[i].getText());
				}
			}
		}
		else
		{
            // Print out all of the AS/400 messages
			AS400Message[] msgs = pcd.getMessageList("qgyolobj");

			for (int i = 0; i < msgs.length; i++)
			{
				System.err.print(msgs[i].getID() + "  ");
				System.err.println(msgs[i].getText());
			}
		}
    }

    /**
     * Returns the count of retrieved messages.
     *
     * @return The count of message queues on the AS/400.
     */
    public int getMessageQueueCount()
    {
        return m_msgQueueList.size();
    }

    /**
     * Returns an AS/400 message queue from the <code>MqMessageQueueList.</code>
     *
     * @param index The position of the desired message queue in the list.
     * @return The <code>MqMessageQueue</code> at the specified index.
     */
    public MqMessageQueue getMessageQueue(int index)
    {
        return (MqMessageQueue)m_msgQueueList.elementAt(index);
    }

	private static boolean debugFlag = true;
	private static void debug(String text)
	{
		if (debugFlag)
			System.out.println("MqMessageQueueList: " + text);
	}
}
