//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

package com.ibm.as400.opnav.MsgQueueSample5;

// Import the entire opnav package
import com.ibm.as400.opnav.*;

// Import the PDML runtime
import com.ibm.as400.ui.framework.java.*;

// Import the access classes we need
import com.ibm.as400.access.AS400;
import com.ibm.as400.access.QSYSObjectPathName;

// Miscellaneous Sun classes
import java.awt.*;
import javax.swing.*;
import java.text.MessageFormat;

/**
 * The <code>TasksManager</code> implementation for this plug-in.
 * <p>
 * A <code>TasksManager</code> implementation provides Operations
 * Navigator with lists of possible user tasks for the plug-in's objects.
 * If the user selects a task, <code>TaskManager</code> is called
 * to carry out the task.
 * See <strong>com.ibm.as400.opnav.TasksManager.</strong>
 * <p>
 * <code>MqTasksManager</code> is invoked whenever the Navigator
 * needs to display the task view for a message queue container,
 * or a message object in the container.
 */
public class MqTasksManager implements TasksManager
{
    /**
     * The selected object.
     */
    ObjectName    m_objectName;

    /**
     * The owning frame for any dialog windows displayed
     * while processing a task.
     */
    Frame           m_owner;

    // Use the ResourceLoader for loading MRI from the sample resource bundle
    private static ResourceLoader  m_loader = new ResourceLoader();

    // For loading images
    private static Toolkit m_toolkit = Toolkit.getDefaultToolkit();

    static
    {
        // Set up to load MRI.  We do this in the static initializer for
        // the class so that we only load the resource bundle once.
        m_loader.setResourceName("com.ibm.as400.opnav.MsgQueueSample5.MessageQueueGUI");

        // Set the service log file name in production
//      try { Monitor.setLogFileName(Monitor.SHARED_LOG_FILE_NAME); }
        try { Monitor.setLogFileName(null); }
        catch (java.io.IOException e) {}
    }

    /**
     * Identifies the object in a list on which the user may wish
     * to perform some task.
     *
     * Called immediately after this object is instantiated.
     * See <strong>com.ibm.as400.opnav.TasksManager.</strong>
     *
     * @param objectName    An object name.
     */
    public void initialize(ObjectName objectName)
    {
        m_objectName = objectName;
    }

    /**
     * Returns a list of tasks which are applicable to the specified objects.
     *
     * Called when Operations Navigator needs to display the available
     * tasks for an object in a list of system objects.
     * See <strong>com.ibm.as400.opnav.TasksManager.</strong>
     *
     * @param flags  The constants which indicate the type of tasks requested.
     * @param verb   A language-independent verb which identifies the main task
     * to which this TasksManager should add cascading tasks. If the
     * CASCADING_TASKS flag is not set this parameter will be null.
     * @return    A list of task descriptors, each of which describes one task.
     * If no tasks are defined on the objects, <code>queryTasks</code> should
     * return an array of length zero.
     */
    public TaskDescriptor[] queryTasks(int flags, String verb)
    {
        // Default is to return an array with zero elements
        TaskDescriptor[] tasks = new TaskDescriptor[0];

        // Determine what type of object(s) we're dealing with
        String objectType = null;

        try { objectType = m_objectName.getObjectType(); }
        catch (ObjectNameException e)
        {
            // Error unlikely in production
            Monitor.logThrowable(e);

            return tasks;
        }

        // Handle the normal task view case
        if (objectType.equals("MessageQueue5"))
        {
            // This is a message queue container.  We have stuff to add to
            // the custom section of the task view.

            if ((flags & CUSTOM_TASKS) == CUSTOM_TASKS)
            {
                // Create an array to contain the returned descriptor
                tasks = new TaskDescriptor[1];

                // Load the image
                ImageIcon icon = new ImageIcon();
                icon.setImage(m_toolkit.getImage(getClass().getResource("MqNewMsg.gif")));

                // Create the descriptor for "Message"
                TaskDescriptor d;

                d = new TaskDescriptor(1);
                d.setName(m_loader.getString("message.text.newmsgtaskname"));
                d.setDescription(m_loader.getString("message.text.newmsgtaskdesc"));
                d.setVerb("IBM.MessageQueueSample5.NewMsg");
//                d.setIcon(icon);
                d.setIcon("MqNewMsg.gif");
                d.setEnabled(true);

                // Set it in the array
                tasks[0] = d;
            }
        }

        // Return tasks, if any
        return tasks;
    }

    /**
     * Informs the <code>TasksManager</code> implementation that a particular
     * task has been selected by the user.
     * <p>
     * The implementation should respond to the request by displaying a
     * UI panel or performing some task on behalf of the user.
     *
     * @param task  The identifier of the task specified in the
     * <code>TaskDescriptor</code> that was returned by <code>queryTasks</code>.
     * @param owner   The frame to be specified as the owner of any dialog panels that are displayed.
     */
    public void taskSelected(int task, Frame owner)
    {
        // Set the owning frame for later use
        m_owner = owner;

        switch (task)
        {
            case 1:
                // Perform the Add task
                promptNewMessage();
                break;

            default:
                // Error unlikely in production
                Monitor.logError("MqTasksManager: Invalid task ID: " + task);
                break;
        }
    }

    private void promptNewMessage()
    {
        // If an error occurs we need the name of the AS/400 we're trying to connect to
        String systemName = "UNKNOWN";

        // Get the name of the AS/400 associated with this container object
        try { systemName = m_objectName.getSystemName(); }
        catch (ObjectNameException e) {}

        // Get the Toolbox object for the container's parent AS/400 system
        AS400 as400 = null;

        try { as400 = (AS400)m_objectName.getSystemObject(); }
        catch (ObjectNameException e)
        {
            // Log the error
            Monitor.logThrowable(e);

            // Load the error message text
            String msg = m_loader.getString("error.text.connection.info");

            // Substitution text is the system name
            Object[] args = { systemName };

            // Format the error message
            String usrMsg = MessageFormat.format(msg, args);

            // Display it to the user
            MessageBoxDialog.showMessageDialog(m_owner,
                                               usrMsg,
                                               m_loader.getString("error.title.msgbox"),
                                               JOptionPane.WARNING_MESSAGE);

            return;
        }

        // Get the name of the selected message queue
        String queueName = null;
        try { queueName = m_objectName.getDisplayName(); }
        catch (ObjectNameException e)
        {
            // Error unlikely in production
            Monitor.logThrowable(e);
            return;
        }

        // Instantiate the data bean for the New Message panel
        QSYSObjectPathName queuePath = new QSYSObjectPathName("QUSRSYS", queueName, "MSGQ");
        MqNewMessageBean msgBean = new MqNewMessageBean(as400, queuePath.getPath(), m_owner);
        msgBean.load();

        // Set up to pass to PanelManager
        DataBean[] dataBeans = { msgBean };

        // Create the panel
        PanelManager pm = null;
        try
        {
            pm = new PanelManager("com.ibm.as400.opnav.MsgQueueSample5.MessageQueueGUI",
                                  "IDD_MSGQ_ADD",
                                  dataBeans,
                                  m_owner);
        }
        catch (DisplayManagerException e)
        {
            // Tell the user an error occurred
            e.displayUserMessage(m_owner);

            return;
        }

        // Display the panel (we give up control here)
        pm.setVisible(true);

        // If no new message was created, simply return
        if (!msgBean.m_actionPerformed)
            return;

        // Refresh the list view to show the new added message
        try { new UIServices().refreshList(m_owner, m_loader.getString("message.text.newmsg")); }
        catch (UIServicesException e)
        {
            // Error unlikely
            Monitor.logThrowable(e);
        }
    }
}
