//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

// We avoid any references to the opnav package in case we want to
// use this class in a standalone application or applet

package com.ibm.as400.opnav.MsgQueueSample6;

// Import the access classes we need
import com.ibm.as400.access.MessageQueue;
import com.ibm.as400.access.QueuedMessage;

// Miscellaneous Sun classes
import java.util.Calendar;

/**
 * An object which represents a queued message on the AS/400.
 * The message's attributes are stored as member fields.  A gettor
 * method is defined for each attribute.
 */
public class MqMessage extends Object
{
    // Define the attributes available on com.ibm.as400.access.AS400Message
    private Calendar        m_date;
    private String          m_defaultReply;
    private String          m_fileName;
    private String          m_help;
    private String          m_ID;
    private String          m_libraryName;
    private String          m_path;
    private int             m_severity;
    private byte[]          m_substitutionData;
    private String          m_text;
    private int             m_type;

    // Define the attributes available on com.ibm.as400.access.QueuedMessage (a subclass)
    private String          m_fromProgram;
    private String          m_fromJobName;
    private String          m_fromJobNumber;
    private byte[]          m_key;
    private MessageQueue    m_queue;
    private String          m_replyStatus;
    private String          m_user;

    /**
     * Constructs an <code>MqMessage</code> object.
     * The receiver extracts the message attributes from the <code>QueuedMessage</code>
     * supplied as a parameter.
     *
     * @param qm    An object of type <code>com.ibm.as400.access.QueuedMessage</code>.
     */
    public MqMessage(QueuedMessage qm)
    {
        // Extract the base class attributes
        m_date = qm.getDate();
        m_defaultReply = qm.getDefaultReply();
        // Don't get file name or library name.  These don't work properly
        // for messages copied from other queues or added via New Message.
        // Besides, these fields are not used in the Message Queue Sample.
        //m_fileName = qm.getFileName();
        //m_libraryName = qm.getLibraryName();
        m_help = qm.getHelp();
        m_ID = qm.getID();
        m_path = qm.getPath();
        m_severity = qm.getSeverity();
        m_substitutionData = qm.getSubstitutionData();
        m_text = qm.getText();
        m_type = qm.getType();

        // Extract the subclass attributes
        m_fromProgram = qm.getFromProgram();
        m_fromJobName = qm.getFromJobName();
        m_fromJobNumber = qm.getFromJobNumber();
        m_key = qm.getKey();
        m_queue = qm.getQueue();
        m_replyStatus = qm.getReplyStatus();
        m_user = qm.getUser();
    }

    /**
     * Returns the date and time the message was issued.
       If not set, null will be returned.
     * The valid fields are :
     * <ul><li> Calendar.YEAR
     *     <li> Calendar.MONTH
     *     <li> Calendar.DAY_OF_MONTH
     *     <li> Calendar.HOUR
     *     <li> Calendar.MINUTE
     *     <li> Calendar.SECOND
     * </ul>
     *
     * @return  The date and time the message was issued.
                If not set, null will be returned.
     **/
    public Calendar getDate()
    { return m_date; }

    /**
     * Returns the default reply for the message.
       If not set, null will be returned.
     *
     * @return  The default reply for the message.
       If not set, null will be returned.
     **/
    public String getDefaultReply()
    { return m_defaultReply; }

    /**
      *Returns the message file name.
      *This is the AS/400 file containing the message.
       If not set, null will be returned.
      *
      *@return  The message file name.  If not set, null will be returned.
      **/
    public String getFileName()
    { return m_fileName; }

    /**
      *Returns any message help text.
       If not set, null will be returned.
      *
      * Message formatting characters may appear in the message help
      * and are defined as follows:
      * <UL>
      *   <LI> &N - Force the text to a new line indented to column 2.  If the text
      *             is longer than 1 line, the next lines should be indented to
      *             column 4 until the end of the text or another format control
      *             character is found.
      *   <LI> &P - Force the text to a new line indented to column 6.  If the
      *             text is longer than 1 line, the next lines should start in
      *             column 4 until the end of the text or another format control
      *             character is found.
      *   <LI> &B - Force the text to a new line starting in column 4.  If the
      *             text is longer than 1 line, the next lines should start in
      *             column 6 until the end of the text or another format control
      *             character is found.
      * </UL>
      *
      *
      *@return  The message help text.  If not set, null will be returned.
      **/
    public String getHelp()
    { return m_help; }

    /**
      *Returns the ID for the message.  If not set, null will be returned.
      *
      *@return  The ID for the message. If not set, null will be returned.
      **/
    public String getID()
    { return m_ID; }

    /**
      *Returns the message library name.
      *This is the AS/400 library containing the file and message.   If not set, null will be returned.
      *
      *@return  The message library name. If not set, null will be returned.
      **/
    public String getLibraryName()
    { return m_libraryName; }

    /**
      *Returns the full integrated file system path name of the
      *message file containing the message.
      *
      *@return  The fully-qualified message file name.
      *         If not set, null will be returned.
      **/
    public String getPath()
    { return m_path; }

    /**
      *Returns the message severity.  Severity is between 0 and 99.  If not set, negative one (-1) will be returned.
      *
      *@return  The message severity. If not set, negative one (-1) will be returned.
      **/
    public int getSeverity()
    { return m_severity; }

    /**
      *Returns the message substitution text.
      *This is the unconverted data used to fill in the replacement
      *characters in the message. If not set, null will be returned.
      *
      *@return  The substitution text. If not set, null will be returned.
      **/
    public byte[] getSubstitutionData()
    { return m_substitutionData; }

    /**
      *Returns the message text.
      *The substitution text has already been inserted. If not set, null will be returned.
      *
      *@return  The message text. If not set, null will be returned.
      **/
    public String getText()
    { return m_text; }

    /**
      *Returns the message type. If not set, null will be returned.
      *
      *@return The message type.<br>
      *<ul>
      *<li>   1 :    Completion
      *<li>   2 :    Diagnostic
      *<li>   4 :    Informational
      *<li>   5 :    Inquiry
      *<li>   6 :    Sender's copy
      *<li>   8 :    Request
      *<li>  10 :    Request with prompting
      *<li>  14 :    Notify
      *<li>  15 :    Escape
      *<li>  21 :    Reply, not validity checked
      *<li>  22 :    Reply, validity checked
      *<li>  23 :    Reply, message default used
      *<li>  24 :    Reply, system default used
      *<li>  25 :    Reply, from system reply list
      *</ul>
      * If not set, null will be returned.
      **/
    public int getType()
    { return m_type; }

    /**
     * Returns the program from which the message originated.
     *
     * @return the program from which the message originated.
    **/
    public String getFromProgram()
    { return m_fromProgram; }

    /**
     * Returns the name of the job from which the the message originated.
     *
     * @return The name of the job from which the the message originated.
    **/
    public String getFromJobName()
    { return m_fromJobName; }

    /**
     * Returns the number of the job from which the the message originated.
     *
     * @return The number of the job from which the the message originated.
    **/
    public String getFromJobNumber()
    { return m_fromJobNumber; }

    /**
     * Returns the message key.
     *
     * @return The message key.
    **/
    public byte[] getKey()
    { return m_key; }

    /**
     * Returns the message queue.
     *
     * @return The message queue.
    **/
    public MessageQueue getQueue()
    { return m_queue; }

    /**
     * Returns the message reply status.
     *
     * @return The message reply status.
    **/
    public String getReplyStatus()
    { return m_replyStatus; }

    /**
     * Returns the user of the job from which the message originated.
     *
     * @return The user of the job from which the message originated.
    **/
    public String getUser()
    { return m_user; }
}
