//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

package com.ibm.as400.opnav.MsgQueueSample6;

// Import the entire OpNav package
import com.ibm.as400.opnav.*;

// Import the Toolbox objects that we need
import com.ibm.as400.access.AS400;
import com.ibm.as400.access.AS400Exception;

// For message formatting
import java.text.MessageFormat;

/**
 * The <code>ListManager</code> implementation for lists of queued messages.
 * <p>
 * A <code>ListManager</code> implementation provides list data for
 * Operations Navigator tree and list views.  Its methods are called
 * whenever the Navigator needs to display the contents of a container.
 * See <strong>com.ibm.as400.opnav.ListManager.</strong>
 * <p>
 * <code>MqMessagesListManager</code> is instantiated by
 * <code>MqListManager</code> whenever the Navigator needs to
 * display a list of messages for a particular message queue.
 */
public class MqMessagesListManager implements ListManager, SortingListManager
{
    /**
     * Identifies the container in the OpNav hierarchy whose contents are to be displayed.
     */
    ObjectName      m_containerName;

    /**
     * Tracks the status of the list.
     */
    int             m_listStatus = LIST_CLOSED;

    /**
     * The object that actually obtains the queued messages from the AS/400.
     */
    MqMessageQueue  m_messageQueue;

    /**
     * The error message to be displayed in case of error.
     */
	String          m_errorMessage;

    /**
     * The ID of the column on which to sort the list data.
     */
    int             m_columnID;

    /**
     * The order in which to sort the list data, if requested by the user.
     */
    int             m_sortOrder;

    // Use the ResourceLoader for loading MRI from the sample resource bundle
    static private ResourceLoader  m_loader = new ResourceLoader();

    static
    {
        // Set up to load MRI.  We do this in the static initializer for
        // the class so that we only load the resource bundle once.
        m_loader.setResourceName("com.ibm.as400.opnav.MsgQueueSample6.MessageQueueGUI");

        // Set the service log file name in production
//      try { Monitor.setLogFileName(Monitor.SHARED_LOG_FILE_NAME); }
        try { Monitor.setLogFileName(null); }
        catch (java.io.IOException e) {}
    }

    /**
     * Identifies the container to be enumerated.
     *
     * Called immediately after the receiver is instantiated.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param container  The container whose contents the user wishes to display.
     */
    public void initialize(ObjectName container)
    {
        // Save the name of the list to be populated
        m_containerName = container;
    }

    /**
     * Prepares the list for enumeration.
     *
     * Called when the user initially selects a container.
     * Once it has been called, <code>getStatus</code> should return a
     * value other than <code>LIST_CLOSED</code>.
     * <p>
     * <code>open</code> is called on a data thread so that long running
     * operations will not degrade the performance of the user interface.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     */
    public void open()
    {
        // If an error occurs we need the name of the AS/400 we're trying to connect to
        String systemName = "UNKNOWN";

        try { systemName = m_containerName.getSystemName(); }
        catch (ObjectNameException e) {}

        // Get the Toolbox object for the container's parent AS/400 system
        AS400 as400 = null;

        try { as400 = (AS400)m_containerName.getSystemObject(); }
        catch (ObjectNameException e)
        {
            // Log the error
            Monitor.logThrowable(e);

            // Load the error message text
            String msg = m_loader.getString("error.text.connection.info");

            // Substitution text is the system name
            Object[] args = { systemName };

            // Store the formatted error message
            m_errorMessage = MessageFormat.format(msg, args);

            // Indicate that an error occurred
            m_listStatus = LIST_ERROR;
            return;
        }

        // Get the name of the selected message queue
        String queueName = null;

        try { queueName = m_containerName.getDisplayName(); }
        catch (ObjectNameException e)
        {
            // Log the error
            Monitor.logThrowable(e);

            // Load the error message text
            String msg = m_loader.getString("error.text.retrieve.msgs");

            // Substitution text is the system name
            Object[] args = { systemName };

            // Store the formatted error message
            m_errorMessage = MessageFormat.format(msg, args);

            // Indicate that an error occurred
            m_listStatus = LIST_ERROR;
            return;
        }

        // Create a message queue object
        m_messageQueue = new MqMessageQueue(queueName, "QUSRSYS", as400);

        // Set sort parameters if required
        if (m_columnID > 0)
            m_messageQueue.setSortParameters((m_columnID == 1 ? "TEXT" : "DATE"),
                                             (m_sortOrder == SORT_ASCENDING ? "ASCENDING" : "DESCENDING"));

        // Retrieve the messages for this message queue
        try { m_messageQueue.loadAS400Messages(); }
        catch (AS400Exception e)
        {
            // Check if the message queue exists
            String msgID = e.getAS400Message().getID();

            if (msgID != null && msgID.equals("CPF2403"))
            {
                // Load the error message text
                String msg = m_loader.getString("error.text.nomsgq");

                // Substitution text is the user name and the system name
                Object[] args = { queueName, systemName };

                // Store the formatted error message
                m_errorMessage = MessageFormat.format(msg, args);
            }
            else
            {
                // Log the error
                Monitor.logThrowable(e);

                // Load the error message text
                String msg = m_loader.getString("error.text.retrieve.msgs");

                // Substitution text is the system name
                Object[] args = { systemName };

                // Store the formatted error message
                m_errorMessage = MessageFormat.format(msg, args);
            }

            // Indicate that an error occurred
            m_listStatus = LIST_ERROR;
            return;
        }
        catch (Exception e)
        {
            // Log the error
            Monitor.logThrowable(e);

            // Load the error message text
            String msg = m_loader.getString("error.text.retrieve.msgs");

            // Substitution text is the system name
            Object[] args = { systemName };

            // Store the formatted error message
            m_errorMessage = MessageFormat.format(msg, args);

            // Indicate that an error occurred
            m_listStatus = LIST_ERROR;
            return;
        }

        // Everything worked, so indicate success
        m_listStatus = LIST_COMPLETE;
    }

    /**
     * Returns an error message.
     *
     * Called after <code>getListStatus</code> has returned
     * a value of <code>LIST_ERROR</code>.  The returned message should be
     * in a form suitable for displaying to the user.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @return  A user error message, or <code>null</code> if no error message is available.
     */
    public String getErrorMessage()
    {
        // We don't need to keep the message around once OpNav
        // has retrieved it and displayed it to the user

        String msg = "";

        if (m_errorMessage != null)
        {
            msg = m_errorMessage;
            m_errorMessage = null;
        }

        return msg;
    }

    /**
     * Returns the current status of the list.
     *
     * Called after the list has been opened to learn
     * whether the open attempt was successful.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @return  A status constant which reflects the current list status.
     */
    public int getStatus()
    {
        // Return the current list status
        return m_listStatus;
    }

    /**
     * Returns the total count of objects in the list.
     * <p>
     * Called after the list has been successfully opened.
     * If <code>getStatus</code> returned a value of
     * <code>LIST_COMPLETE</code>, this method should return
     * the total number of objects in the list.  If <code>getStatus</code>
     * returned <code>LIST_INCOMPLETE</code>, this method should
     * return the number of objects currently known to be in the list.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @return  The number of objects in the list.
     */
    public int getItemCount()
    {
        // Return the count of retrieved messages
        return m_messageQueue.getMessageCount();
    }

    /**
     * Returns the item identifier for a list object.
     * <p>
     * Called repeatedly when populating a view of a container.
     * Long running operations should be avoided.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param  index  The zero-based index of the item for which an
     * item identifier is requested.
     * @return  The item identifier that describes the object at the
     * specified position in the list.
     */
    public ItemIdentifier itemAt(int index)
    {
        // Get the requested message object
        MqMessage msg = m_messageQueue.getMessage(index);

        // Construct the OpNav item identifier
        ItemIdentifier item = new ItemIdentifier(index);
        item.setName(msg.getText());
        item.setType("Message6");

        // Return the new item
        return item;
    }

    /**
     * Returns the attributes of a list object.
     * <p>
     * Called repeatedly when populating a view of a container.
     * Long running operations should be avoided.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param   item  The item identifier for the object whose attributes
     * are to be retrieved.
     * @return  The integer sum of the attribute constants.
     */
    public int getAttributes(ItemIdentifier item)
    {
        // Message objects have properties which can be viewed
        return ListManager.OBJECT_HASPROPERTIES;
    }

    /**
     * Returns the image file name for a list object.
     * <p>
     * Called repeatedly when populating the view of a container.
     * Long running operations should be avoided.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param   item  The item identifier for the object whose image file
     * name is to be retrieved.
     * @param   flags The constants which reflect the status of the object.
     * @return  The name of the image file containing the icon for the object.
     */
    public String getImageFile(ItemIdentifier item, int flags)
    {
        // To be implemented when OpNav becomes pure Java
        return "";
    }

    /**
     * Returns the icon index into the plug-in's resource DLL for a list object.
     * <p>
     * Called repeatedly when populating the view of a container.
     * Long running operations should be avoided.
     *
     * @param   item  The item identifier for the object whose icon index
     * is to be retrieved.
     * @param   flags The constants which reflect the status of the object.
     * @return  The index into the plug-in's resource DLL of the icon for the object.
     */
    public int getIconIndex(ItemIdentifier item, int flags)
    {
        // Return the index of the message icon in the MRI DLL
        return 2;
    }

    /**
     * Returns the column headings for this list.
     *
     * Called after the list has been successfully opened.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @return  A list of <code>ColumnDescriptor</code> objects
     * that describe the column headings for this list.
     */
    public ColumnDescriptor[] getColumnInfo()
    {
        // Create an array of size 2 since there are two columns
        ColumnDescriptor[] descriptors = new ColumnDescriptor[2];

        // Create the descriptors for the message text and date
        descriptors[0] = new ColumnDescriptor(m_loader.getString("column.msg.col1"), 20, 1);
        descriptors[1] = new ColumnDescriptor(m_loader.getString("column.msg.col2"), 25, 2);

        // Return the list of descriptors
        return descriptors;
  }

    /**
     * Returns the data for a list column.
     * <p>
     * Called repeatedly when populating the view of a container.
     * Long running operations should be avoided.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param   item  The item identifier for the object whose column
     * data is to be retrieved.
     * @param   columnID  The numeric ID that identifies the column.
     * @return  The data for the specified column converted to a String.
     */
    public String getColumnData(ItemIdentifier item, int columnID)
    {
        // Get the requested message object
        MqMessage msg = m_messageQueue.getMessage(item.getIndex());

        // Return the data for the requested column
        String sData = "";

        switch (columnID)
        {
            case 1:
                sData = msg.getText();
                break;

            case 2:
                // Convert the Calendar object returned by Message.getDate to a readable string
                sData = msg.getDate().getTime().toString();
                break;

            default:
                sData = "Invalid column ID: " + columnID;
                break;
        }

        return sData;
    }

    /**
     * Returns a cached list object.
     *
     * Called when another interface implementation
     * requires access to the object that represents a
     * resource on the system being managed.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     *
     * @param objName The name which identifies the object to be retrieved.
     * @return The cached list object.
     */
    public Object getListObject(ObjectName objName)
    {
        // Extract the index of the message object from the object name
        int nIndex;

        try { nIndex = objName.getObjectIndex(); }
        catch (ObjectNameException e)
        {
            // Error unlikely in production
            Monitor.logThrowable(e);
            return null;
        }

        // Return the requested message object
        return m_messageQueue.getMessage(nIndex);
    }

    /**
     * Frees all list resources.
     *
     * Called when the user closes a view of a container,
     * or when a refresh of the container's contents is requested.
     * Once it has been called, <code>getStatus</code> should return a
     * value of <code>LIST_CLOSED</code>.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     */
    public void close()
    {
        // Indicate list now closed
        m_listStatus = LIST_CLOSED;

        // Free up the message queue
        m_messageQueue = null;
    }

    /**
     * Provides notification that the Operations Navigator application is terminating.
     *
     * Called when the user closes the application.
     * Provides the implementation with the opportunity to clean
     * up any connection handles and save any persistent data.
     * See <strong>com.ibm.as400.opnav.ListManager.</strong>
     */
    public void prepareToExit()
    {
        // Nothing special needs to be done when OpNav terminates
    }

    /**
     * Indicates whether sorting is enabled for a list.
     *
     * Called when the user initially selects a container of
     * system objects to determine whether the user interface should
     * enable sorting.  See <strong>com.ibm.as400.opnav.SortingListManager.</strong>
     *
     * @return true if the list can be sorted; false otherwise.
     */
    public boolean isSortingEnabled()
    {
        System.out.println("MqMessagesListManager: IN ISSORTINGENABLED!");
        return true;
    }

    /**
     * Sorts the list on the specified table column in a view of system objects.
     *
     * Called when the user has requested a sort of the list items
     * on a given table column.  On return from this method, the user interface will immediately
     * refresh the list by calling <code>ListManager.close</code> followed by
     * <code>ListManager.open</code>.  See <strong>com.ibm.as400.opnav.SortingListManager.</strong>
     *
     * @param columnID	the numeric ID identifying the column to sort on, as specified in the column descriptor.
     * @param sortOrder	SortingListManager.SORT_ASCENDING if the list is to be sorted in ascending order;
     * SortingListManager.SORT_DESCENDING if the list is to be sorted in descending order.
     * @return  true if sorting on the specified column is supported; false otherwise.
     */
    public boolean sortOnColumn(int columnID, int sortOrder)
    {
        System.out.println("MqMessagesListManager: IN SORTONCOLUMN![" + columnID + "," + sortOrder + "]");

        if (columnID == 1)
        {
            m_columnID = columnID;
            m_sortOrder = sortOrder;
            return true;
        }

        return false;
    }
    
    /**
     * Indicates whether multiple sorting is enabled for a list, and which columns
     * it is enabled on.
     *
     * Called when the user initially selects a container of
     * system objects to determine whether the user interface should
     * enable multiple sorting. Return an array including all of the columns
     * that support multiple sorting. If there are none, return an empty array.
     * See <strong>com.ibm.as400.opnav.SortingListManager.</strong>
     *
     * @return  an array of <code>ColumnDescriptor</code> objects
     */
    public ColumnDescriptor[] getSortingColumnInfo()
    {
    	return new ColumnDescriptor[0];
    }
    
    /**
     * Sorts the lists on the specified table columns in a view of system objects.
     *
     * Called when the user has requested a sort of the list items
     * on given table columns.  On return from this method, the user interface will immediately
     * refresh the list by calling <code>ListManager.close</code> followed by
     * <code>ListManager.open</code>. It is assumed that the columns will be sorted
     * in ascending order.  See <strong>com.ibm.as400.opnav.SortingListManager.</strong>
     *
     * @param columns The array of the ColumnDescriptors of the columns you want sorted.
     * @return  true if sorting on the specified columns is supported; false otherwise.
     */    
    public boolean sortOnColumns(ColumnDescriptor[] columns)
    {
    	return false;
    }
}
